<?php 
/* -------------------------------------------------------------
This file is part of DOAP (Dave's Own Access Protocol)

DOAP is (C) Copyright 2011 PurplePixie Systems

DOAP is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DOAP is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DOAP.  If not, see www.gnu.org/licenses

For more information see http://www.purplepixie.org/doap
-------------------------------------------------------------- */

/**
 * DOAP Server Module - Dave's Own Access Protocol
 * @author David Cutting
 * @package DOAP
 * @copyright 2010 PurplePixie Systems
 * @version 0.01
**/

// First we check for xmlCreate and try to include it if not found
if (!class_exists("XMLCreate"))
{
	$tryfile = dirname(__FILE__)."/"."xmlcreate.inc.php";
	if (file_exists($tryfile)) include_once($tryfile);
}

/**
 * DOAP Server Class
 * @package DOAP
**/
class DOAP_Server
{

/**
 * Keep list of registered functions and associated data
**/
private $functions = array();

/**
 * Register a function
 * @param string $funcname Name of function to register
 * @param int $param_min Minimum parameters for function (optional, default not enforced)
 * @param int $param_max Maximum parameters for function (optional, default not enforced)
**/
function Register($funcname,$param_min=-1, $param_max=-1)
{
	$this->functions[$funcname]['name']=$funcname;
	$this->functions[$funcname]['min']=$param_min;
	$this->functions[$funcname]['max']=$param_max;
}

/**
 * Generate a DOAP Error in XML
 * @param int $code Error Code Number
 * @param string $desc Error Description
 * @param bool $outputxml Output XML to Screen (as well as return) (optional, default false)
 * @return string XML output
**/
function Error($code,$desc,$outputxml=true)
{
	$xml = new XMLCreate();
	$xml -> startElement("ERROR");
	$xml -> charElement("CODE",$code,0,false,true);
	$xml -> charElement("DESC",$desc,0,false,true);
	$xml -> endElement("ERROR");
	
	if ($outputxml)
	{
		@header("Content-type: text/xml");
		echo "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n";
		echo $xml->xml;
	}
	
	return $xml -> xml;
}

/**
 * Add an array to the XML output (may be called recursively for arrays in an array object)
 * @param array $array Array to parse
 * @param string &$xml Pointer to output XML
**/
function AddArray($array,&$xml)
{
	$xml -> startElement("ARRAY");
	foreach($array as $element)
	{
		if (!is_array($element)) $xml->charElement("ELEMENT",$element,0,false,true);
		else $this->AddArray($element,$xml);
	}
	$xml->endElement("ARRAY");
}

/**
 * Process any DOAP Requests
 * @param bool $outputxml Output XML to the screen (optional, default true)
 * @return string DOAP XML Output
**/
function Process($outputxml=true)
{
	if (!class_exists("xmlCreate"))
	{
		$xml="<ERROR>\n <CODE>208</CODE>\n <DESC>xmlCreate Library Not Found</DESC>\n</ERROR>\n";
		if ($outputxml)
		{
			@header("Content-type: text/xml");
			echo "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n";
			echo $xml;
		}
		return $xml;
	}
	
	if (isset($_REQUEST['f'])) $func=$_REQUEST['f'];
	else return $this->Error(200,"No function specified",$outputxml);
	
	if (!isset($this->functions[$func])) return $this->Error(201,"Illegal function specified: ".$func,$outputxml);
	
	if (isset($_REQUEST['p'])) $params=$_REQUEST['p'];
	else $params=array();
	
	if (!is_array($params)) $params=array($params);
	
	if (!function_exists($func)) return $this->Error(202,"Non-existant function specified: ".$func,$outputxml);
	
	if ( ($this->functions[$func]['min']>0) && (sizeof($params)<$this->functions[$func]['min']) )
		return $this->Error(203,"Too few parameters for function ".$func." (needs ".$this->functions[$func]['min'].")",$outputxml);
	
	if ( ($this->functions[$func]['max']>=0) && (sizeof($params)>$this->functions[$func]['max']) )
		return $this->Error(203,"Too many parameters for function ".$func." (max ".$this->functions[$func]['max'].")",$outputxml);
	
	switch(sizeof($params)) // Oh dear so messy but without using eval() and the performance hit the only way I think
	{
		case 0: $res = $func();
		break;
		case 1: $res = $func($params[0]);
		break;
		case 2: $res = $func($params[0],$params[1]);
		break;
		case 3: $res = $func($params[0],$params[1],$params[2]);
		break;
		case 4: $res = $func($params[0],$params[1],$params[2],$params[3]);
		break;
		case 5: $res = $func($params[0],$params[1],$params[2],$params[3],$params[4]);
		break;
		case 6: $res = $func($params[0],$params[1],$params[2],$params[3],$params[4], $params[5]);
		break;
		case 7: $res = $func($params[0],$params[1],$params[2],$params[3],$params[4], $params[5],
			$params[6]);
		break;
		case 8: $res = $func($params[0],$params[1],$params[2],$params[3],$params[4], $params[5],
			$params[6], $params[7]);
		break;
		case 9: $res = $func($params[0],$params[1],$params[2],$params[3],$params[4], $params[5],
			$params[6], $params[7], $params[8]);
		break;
		case 10: $res = $func($params[0],$params[1],$params[2],$params[3],$params[4], $params[5],
			$params[6], $params[7], $params[8], $params[9]);
		break;
		default: return $this->Error(204,"Parameters in excess of DOAP Limit (10)",$outputxml);
	}
	
	$xml = new xmlCreate();

	if (!is_array($res)) $xml->charElement("RESULT",$res,0,false,true);
	else $this->AddArray($res,$xml);
	
	if ($outputxml)
	{
		@header("Content-type: text/xml");
		echo "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n";
		echo $xml->xml;
	}
	
	return $xml->xml;
}

}
?>